@@ -46,6 +46,29 @@ gfInd <- function( param ) {
    return( llGrads )
 }
 
+## log likelihood function with gradients as attributes
+llfGrad <- function( param ) {
+   mu <- param[ 1 ]
+   sigma <- param[ 2 ]
+   N <- length( x )
+   llValue <- -0.5 * N * log( 2 * pi ) - N * log( sigma ) -
+      0.5 * sum( ( x - mu )^2 / sigma^2 )
+   attributes( llValue )$gradient <- c( sum( ( x - mu ) / sigma^2 ),
+      - N / sigma + sum( ( x - mu )^2 / sigma^3 ) )
+   return( llValue )
+}
+
+## log likelihood function with gradients as attributes (individual observations)
+llfGradInd <- function( param ) {
+   mu <- param[ 1 ]
+   sigma <- param[ 2 ]
+   llValues <- -0.5 * log( 2 * pi ) - log( sigma ) -
+      0.5 * ( x - mu )^2 / sigma^2
+   attributes( llValues )$gradient <- cbind( ( x - mu ) / sigma^2,
+      - 1 / sigma + ( x - mu )^2 / sigma^3 )
+   return( llValues )
+}
+
 ## function to calculate analytical Hessians
 hf <- function( param ) {
    mu <- param[ 1 ]
@@ -60,6 +83,43 @@ hf <- function( param ) {
    return( llHess )
 }
 
+## log likelihood function with gradients and Hessian as attributes
+llfGradHess <- function( param ) {
+   mu <- param[ 1 ]
+   sigma <- param[ 2 ]
+   N <- length( x )
+   llValue <- -0.5 * N * log( 2 * pi ) - N * log( sigma ) -
+      0.5 * sum( ( x - mu )^2 / sigma^2 )
+   attributes( llValue )$gradient <- c( sum( ( x - mu ) / sigma^2 ),
+      - N / sigma + sum( ( x - mu )^2 / sigma^3 ) )
+   attributes( llValue )$hessian <- matrix( c(
+      N * ( - 1 / sigma^2 ),
+      sum( - 2 * ( x - mu ) / sigma^3 ),
+      sum( - 2 * ( x - mu ) / sigma^3 ),
+      N / sigma^2 + sum( - 3 * ( x - mu )^2 / sigma^4 ) ),
+      nrow = 2, ncol = 2 )
+   return( llValue )
+}
+
+## log likelihood function with gradients as attributes (individual observations)
+llfGradHessInd <- function( param ) {
+   mu <- param[ 1 ]
+   sigma <- param[ 2 ]
+   N <- length( x )
+   llValues <- -0.5 * log( 2 * pi ) - log( sigma ) -
+      0.5 * ( x - mu )^2 / sigma^2
+   attributes( llValues )$gradient <- cbind( ( x - mu ) / sigma^2,
+      - 1 / sigma + ( x - mu )^2 / sigma^3 )
+   attributes( llValues )$hessian <- matrix( c(
+      N * ( - 1 / sigma^2 ),
+      sum( - 2 * ( x - mu ) / sigma^3 ),
+      sum( - 2 * ( x - mu ) / sigma^3 ),
+      N / sigma^2 + sum( - 3 * ( x - mu )^2 / sigma^4 ) ),
+      nrow = 2, ncol = 2 )
+   return( llValues )
+}
+
+
 # start values
 startVal <- c( mu = 0, sigma = 1 )
 
@@ -75,7 +135,7 @@ hessian( ml )
 logLik( ml )
 maximType( ml )
 nIter( ml )
-nObs( ml )
+try( nObs( ml ) )
 nParam( ml )
 returnCode( ml )
 returnMessage( ml )
@@ -83,7 +143,9 @@ vcov( ml )
 logLik( summary( ml ) )
 mlInd <- maxLik( llfInd, start = startVal )
 summary( mlInd )
-all.equal( ml, mlInd )
+all.equal( ml[ ], mlInd[ -11 ] )
+mlInd[ 11 ]
+nObs( mlInd )
 
 # with analytical gradients
 mlg <- maxLik( llf, gf, start = startVal )
@@ -91,12 +153,38 @@ summary( mlg )
 all.equal( ml, mlg )
 mlgInd <- maxLik( llfInd, gfInd, start = startVal )
 all.equal( mlInd, mlgInd )
-all.equal( mlg, mlgInd )
+all.equal( mlg[ ], mlgInd[ -11 ] )
+mlgInd[ 11 ]
+
+# with analytical gradients as attribute
+mlG <- maxLik( llfGrad, start = startVal )
+all.equal( mlG, mlg, tolerance = 1e-10 )
+all.equal( mlG$gradient, gf( coef( mlG ) ), check.attributes = FALSE )
+mlGInd <- maxLik( llfGradInd, start = startVal )
+all.equal( mlGInd, mlgInd, tolerance = 1e-10 )
+all.equal( mlGInd$gradient, colSums( gfInd( coef( mlGInd ) ) ),
+   check.attributes = FALSE )
+all.equal( mlGInd$gradientObs, gfInd( coef( mlGInd ) ),
+   check.attributes = FALSE )
+
+# with analytical gradients as argument and attribute
+mlgG <- maxLik( llfGrad, gf, start = startVal )
+all.equal( mlgG, mlg, tolerance = 1e-10 )
+all.equal( mlgG, mlG, tolerance = 1e-10 )
 
 # with analytical gradients and Hessians
 mlgh <- maxLik( llf, gf, hf, start = startVal )
 all.equal( mlg, mlgh )
 
+# with analytical gradients and Hessian as attribute
+mlGH <- maxLik( llfGradHess, start = startVal )
+all.equal( mlGH, mlgh, tolerance = 1e-10 )
+
+# with analytical gradients and Hessian as argument and attribute
+mlgGhH <- maxLik( llfGradHess, gf, hf, start = startVal )
+all.equal( mlgGhH, mlgh, tolerance = 1e-10 )
+all.equal( mlgGhH, mlGH, tolerance = 1e-10 )
+
 
 ## BHHH method
 mlBHHH <- try( maxLik( llf, start = startVal, method = "BHHH" ) )
@@ -116,13 +204,20 @@ hessian( mlBHHH )
 logLik( mlBHHH )
 maximType( mlBHHH )
 nIter( mlBHHH )
-nObs( mlBHHH )
 nParam( mlBHHH )
 returnCode( mlBHHH )
 returnMessage( mlBHHH )
 vcov( mlBHHH )
 logLik( summary( mlBHHH ) )
-all.equal( ml, mlBHHH )
+all.equal( ml[ ], mlBHHH[ -11 ] )
+mlBHHH[ 11 ]
+nObs( mlBHHH )
+# final Hessian = usual Hessian
+mlBhhhH <- maxLik( llfInd, start = startVal, method = "BHHH", 
+   finalHessian = TRUE )
+all.equal( mlBhhhH, mlBHHH )
+hessian( mlBhhhH ) 
+summary( mlBhhhH ) 
 
 # with analytical gradients
 mlgBHHH <- try( maxLik( llf, gf, start = startVal, method = "BHHH" ) )
@@ -137,13 +232,119 @@ x <- xSaved
 mlgBHHH <- maxLik( llfInd, gfInd, start = startVal, method = "BHHH" )
 summary( mlgBHHH )
 all.equal( mlBHHH, mlgBHHH )
-all.equal( mlg, mlgBHHH )
+all.equal( mlg[ ], mlgBHHH[ -11 ] )
+mlgBHHH[ 11 ]
 mlgBHHH2 <- maxLik( llf, gfInd, start = startVal, method = "BHHH" )
 all.equal( mlgBHHH, mlgBHHH2 )
+# final Hessian = usual Hessian
+mlgBhhhH <- maxLik( llf, gfInd, start = startVal, method = "BHHH", 
+   finalHessian = TRUE )
+all.equal( mlgBhhhH, mlBhhhH )
+all.equal( mlgBhhhH, mlgBHHH )
+hessian( mlgBhhhH ) 
+
+# with analytical gradients as attribute
+try( maxLik( llfGrad, start = startVal, method = "BHHH" ) )
+x <- xSaved[1]
+try( maxLik( llfGrad, start = startVal, method = "BHHH" ) )
+try( maxLik( llfGradInd, start = startVal, method = "BHHH" ) )
+x <- xSaved[1:2]
+try( maxLik( llfGrad, start = startVal, method = "BHHH" ) )
+try( maxLik( llfGradInd, start = startVal, method = "BHHH" ) )
+x <- xSaved
+mlGBHHH <- maxLik( llfGradInd, start = startVal, method = "BHHH" )
+all.equal( mlGBHHH, mlgBHHH, tolerance = 1e-10 )
+# final Hessian = usual Hessian
+mlGBhhhH <- maxLik( llfGradInd, start = startVal, method = "BHHH", 
+   finalHessian = TRUE )
+all.equal( mlGBhhhH, mlgBhhhH )
+
+# with analytical gradients as argument and attribute
+mlgGBHHH <- maxLik( llfGradInd, gfInd, start = startVal, method = "BHHH" )
+all.equal( mlgGBHHH, mlgBHHH, tolerance = 1e-10 )
+all.equal( mlgGBHHH, mlGBHHH, tolerance = 1e-10 )
 
 # with unused Hessian
 mlghBHHH <- maxLik( llfInd, gfInd, hf, start = startVal, method = "BHHH" )
 all.equal( mlgBHHH, mlghBHHH )
+# final Hessian = usual Hessian
+mlghBhhhH <- maxLik( llfInd, gfInd, hf, start = startVal, method = "BHHH", 
+   finalHessian = TRUE )
+all.equal( mlghBhhhH, mlghBHHH )
+all.equal( mlghBhhhH, mlgBhhhH )
+
+# with unused Hessian as attribute
+mlGHBHHH <- maxLik( llfGradHessInd, start = startVal, method = "BHHH" )
+all.equal( mlGHBHHH, mlghBHHH, tolerance = 1e-10 )
+# final Hessian = usual Hessian
+mlGHBhhhH <- maxLik( llfGradHessInd, start = startVal, method = "BHHH", 
+   finalHessian = TRUE )
+all.equal( mlGHBhhhH, mlghBhhhH )
+
+# with analytical gradients and Hessian as argument and attribute
+mlgGhHBHHH <- maxLik( llfGradHessInd, gfInd, hf, start = startVal, method = "BHHH" )
+all.equal( mlgGhHBHHH, mlghBHHH, tolerance = 1e-10 )
+all.equal( mlgGhHBHHH, mlGHBHHH, tolerance = 1e-10 )
+
+
+### BFGS-YC method
+mlBFGSYC <- maxLik( llf, start = startVal, method = "bfgsr" )
+print( mlBFGSYC )
+summary( mlBFGSYC )
+activePar( mlBFGSYC )
+AIC( mlBFGSYC )
+coef( mlBFGSYC )
+condiNumber( mlBFGSYC )
+hessian( mlBFGSYC )
+logLik( mlBFGSYC )
+maximType( mlBFGSYC )
+nIter( mlBFGSYC )
+try( nObs( mlBFGSYC ) )
+nParam( mlBFGSYC )
+returnCode( mlBFGSYC )
+returnMessage( mlBFGSYC )
+vcov( mlBFGSYC )
+logLik( summary( mlBFGSYC ) )
+all.equal( ml[-c(5,6,9,10)], mlBFGSYC[-c(5,6,9,10)] )
+mlIndBFGSYC <- maxLik( llfInd, start = startVal, method = "BFGSR" )
+summary( mlIndBFGSYC )
+all.equal( mlBFGSYC[ -9 ], mlIndBFGSYC[ -c(9,11) ] )
+mlIndBFGSYC[ 11 ]
+nObs( mlIndBFGSYC )
+
+# with analytical gradients
+mlgBFGSYC <- maxLik( llf, gf, start = startVal, method = "BFGSR" , print.level=1)
+summary(mlgBFGSYC)
+all.equal( mlBFGSYC, mlgBFGSYC )
+mlgIndBFGSYC <- maxLik( llfInd, gfInd, start = startVal,
+   method = "BFGSR" )
+all.equal( mlIndBFGSYC, mlgIndBFGSYC )
+all.equal( mlgBFGSYC[ -9 ], mlgIndBFGSYC[ -c(9,11) ] )
+mlgIndBFGSYC[ 11 ]
+
+# with analytical gradients as attribute
+mlGBFGSYC <- maxLik( llfGrad, start = startVal, method = "BFGSR" , print.level=1)
+all.equal( mlGBFGSYC, mlgBFGSYC )
+mlGIndBFGSYC <- maxLik( llfGradInd, start = startVal, method = "BFGSR" )
+all.equal( mlGIndBFGSYC, mlgIndBFGSYC )
+
+# with analytical gradients as argument and attribute
+mlgGBFGSYC <- maxLik( llfGrad, gf, start = startVal, method = "BFGSR" )
+all.equal( mlgGBFGSYC, mlgBFGSYC )
+all.equal( mlgGBFGSYC, mlGBFGSYC )
+
+# with analytical gradients and Hessians
+mlghBFGSYC <- maxLik( llf, gf, hf, start = startVal, method = "BFGSR" )
+all.equal( mlgBFGSYC, mlghBFGSYC )
+
+# with analytical gradients and Hessian as attribute
+mlGHBFGSYC <- maxLik( llfGradHess, start = startVal, method = "BFGSR" )
+all.equal( mlGHBFGSYC, mlghBFGSYC, tolerance = 1e-10 )
+
+# with analytical gradients and Hessian as argument and attribute
+mlgGhHBFGSYC <- maxLik( llfGradHess, gf, hf, start = startVal, method = "BFGSR" )
+all.equal( mlgGhHBFGSYC, mlghBFGSYC )
+all.equal( mlgGhHBFGSYC, mlGHBFGSYC )
 
 
 ## BFGS method
@@ -158,20 +359,18 @@ hessian( mlBFGS )
 logLik( mlBFGS )
 maximType( mlBFGS )
 nIter( mlBFGS )
-nObs( mlBFGS )
 nParam( mlBFGS )
 returnCode( mlBFGS )
 returnMessage( mlBFGS )
 vcov( mlBFGS )
 logLik( summary( mlBFGS ) )
 all.equal( ml, mlBFGS )
-mlIndBFGS <- maxLik( llfInd, start = startVal, method = "BFGS" )
-summary( mlIndBFGS )
-all.equal( mlBFGS, mlIndBFGS )
 # with individual log likelihood values
 mlIndBFGS <- maxLik( llfInd, start = startVal, method = "BFGS" )
 summary( mlIndBFGS )
-all.equal( mlBFGS, mlIndBFGS )
+all.equal( mlBFGS[], mlIndBFGS[-12] )
+mlIndBFGS[12]
+nObs( mlIndBFGS )
 
 # with analytical gradients
 mlgBFGS <- maxLik( llf, gf, start = startVal, method = "BFGS" )
@@ -179,12 +378,33 @@ summary( mlgBFGS )
 all.equal( mlBFGS, mlgBFGS )
 all.equal( mlg, mlgBFGS )
 mlgIndBFGS <- maxLik( llfInd, gfInd, start = startVal, method = "BFGS" )
-all.equal( mlgBFGS, mlgIndBFGS )
+all.equal( mlgBFGS[], mlgIndBFGS[-12] )
+mlgIndBFGS[12]
+
+# with analytical gradients as attribute
+mlGBFGS <- maxLik( llfGrad, start = startVal, method = "BFGS" )
+all.equal( mlGBFGS, mlgBFGS, tolerance = 1e-14 )
+mlGIndBFGS <- maxLik( llfGradInd, start = startVal, method = "BFGS" )
+all.equal( mlGIndBFGS, mlgIndBFGS, tolerance = 1e-14 )
+
+# with analytical gradients as argument and attribute
+mlgGBFGS <- maxLik( llfGrad, gf, start = startVal, method = "BFGS" )
+all.equal( mlgGBFGS, mlgBFGS, tolerance = 1e-14 )
+all.equal( mlgGBFGS, mlGBFGS, tolerance = 1e-14 )
 
 # with unused Hessian
 mlghBFGS <- maxLik( llf, gf, hf, start = startVal, method = "BFGS" )
 all.equal( mlgBFGS, mlghBFGS )
 
+# with analytical gradients and Hessian as attribute
+mlGHBFGS <- maxLik( llfGradHess, start = startVal, method = "BFGS" )
+all.equal( mlGHBFGS, mlghBFGS, tolerance = 1e-14 )
+
+# with analytical gradients and Hessian as argument and attribute
+mlgGhHBFGS <- maxLik( llfGradHess, gf, hf, start = startVal, method = "BFGS" )
+all.equal( mlgGhHBFGS, mlghBFGS, tolerance = 1e-14 )
+all.equal( mlgGhHBFGS, mlGHBFGS, tolerance = 1e-14 )
+
 
 ## NM method
 mlNM <- maxLik( llf, start = startVal, method = "NM" )
@@ -198,7 +418,6 @@ hessian( mlNM )
 logLik( mlNM )
 maximType( mlNM )
 nIter( mlNM )
-nObs( mlNM )
 nParam( mlNM )
 returnCode( mlNM )
 returnMessage( mlNM )
@@ -208,7 +427,9 @@ all.equal( ml, mlNM )
 # with individual log likelihood values
 mlIndNM <- maxLik( llfInd, start = startVal, method = "NM" )
 summary( mlIndNM )
-all.equal( mlNM, mlIndNM )
+all.equal( mlNM[], mlIndNM[-12] )
+mlIndNM[12]
+nObs( mlIndNM )
 
 # with unused analytical gradients
 mlgNM <- maxLik( llf, gf, start = startVal, method = "NM" )
@@ -217,7 +438,19 @@ all.equal( mlNM, mlgNM )
 # with individual log likelihood values and gradients
 mlgIndNM <- maxLik( llfInd, gfInd, start = startVal, method = "NM" )
 summary( mlgIndNM )
-all.equal( mlgNM, mlgIndNM )
+all.equal( mlgNM[], mlgIndNM[-12] )
+mlgIndNM[12]
+
+# with (unused) analytical gradients as attribute
+mlGNM <- maxLik( llfGrad, start = startVal, method = "NM" )
+all.equal( mlGNM, mlgNM, tolerance = 1e-14 )
+mlGIndNM <- maxLik( llfGradInd, start = startVal, method = "NM" )
+all.equal( mlGIndNM, mlgIndNM, tolerance = 1e-14 )
+
+# with analytical gradients as argument and attribute
+mlgGNM <- maxLik( llfGrad, gf, start = startVal, method = "NM" )
+all.equal( mlgGNM, mlgNM, tolerance = 1e-14 )
+all.equal( mlgGNM, mlGNM, tolerance = 1e-14 )
 
 # with unused analytical gradients and Hessian
 mlghNM <- maxLik( llf, gf, hf, start = startVal, method = "NM" )
@@ -236,7 +469,6 @@ hessian( mlSANN )
 logLik( mlSANN )
 maximType( mlSANN )
 nIter( mlSANN )
-nObs( mlSANN )
 nParam( mlSANN )
 returnCode( mlSANN )
 returnMessage( mlSANN )
@@ -246,7 +478,9 @@ all.equal( ml, mlSANN )
 # with individual log likelihood values
 mlIndSANN <- maxLik( llfInd, start = startVal, method = "SANN" )
 summary( mlIndSANN )
-all.equal( mlSANN, mlIndSANN )
+all.equal( mlSANN[], mlIndSANN[-12] )
+mlIndSANN[12]
+nObs( mlIndSANN )
 
 # with unused analytical gradients
 mlgSANN <- maxLik( llf, gf, start = startVal, method = "SANN" )
@@ -255,7 +489,8 @@ all.equal( mlSANN, mlgSANN )
 # with individual log likelihood values and gradients
 mlgIndSANN <- maxLik( llfInd, gfInd, start = startVal, method = "SANN" )
 summary( mlgIndSANN )
-all.equal( mlgSANN, mlgIndSANN )
+all.equal( mlgSANN[], mlgIndSANN[-12] )
+mlgIndSANN[12]
 
 # with unused analytical gradients and Hessian
 mlghSANN <- maxLik( llf, gf, hf, start = startVal, method = "SANN" )
@@ -294,7 +529,6 @@ hessian( mlFix )
 logLik( mlFix )
 maximType( mlFix )
 nIter( mlFix )
-nObs( mlFix )
 nParam( mlFix )
 returnCode( mlFix )
 returnMessage( mlFix )
@@ -310,9 +544,11 @@ all.equal( mlIndFix, mlIndFix3 )
 mlIndFix4 <- maxLik( llfInd, start = startValFix, fixed = 1 )
 all.equal( mlIndFix, mlIndFix4 )
 summary( mlIndFix )
-all.equal( mlFix, mlIndFix )
+all.equal( mlFix[ ], mlIndFix[ -11 ] )
 mlFix[[3]]
 mlIndFix[[3]]
+mlIndFix[ 11 ]
+nObs( mlIndFix )
 
 # with analytical gradients
 mlgFix <- maxLik( llf, gf, start = startValFix, activePar = !isFixed )
@@ -332,7 +568,8 @@ mlIndFix[[3]]
 mlgIndFix[[3]]
 mlIndFix[[4]]
 mlgIndFix[[4]]
-all.equal( mlgFix, mlgIndFix )
+all.equal( mlgFix[ ], mlgIndFix[ -11 ] )
+mlgIndFix[ 11 ]
 
 # with analytical gradients and Hessians
 mlghFix <- maxLik( llf, gf, hf, start = startValFix, activePar = !isFixed )
@@ -365,17 +602,18 @@ hessian( mlFixBHHH )
 logLik( mlFixBHHH )
 maximType( mlFixBHHH )
 nIter( mlFixBHHH )
-nObs( mlFixBHHH )
 nParam( mlFixBHHH )
 returnCode( mlFixBHHH )
 returnMessage( mlFixBHHH )
 vcov( mlFixBHHH )
 logLik( summary( mlFixBHHH ) )
-all.equal( mlFix[ -c( 5, 6, 9, 10 ) ], mlFixBHHH[ -c( 5, 6, 9, 10 ) ] )
+all.equal( mlFix[ -c( 5, 6, 9, 10 ) ], mlFixBHHH[ -c( 5, 6, 9, 10, 11 ) ] )
 mlFix[[ 3 ]]
 mlFixBHHH[[ 3 ]]
 mlFix[[ 4 ]]
 mlFixBHHH[[ 4 ]]
+mlFixBHHH[ 11 ]
+nObs( mlFixBHHH )
 
 # with analytical gradients
 mlgFixBHHH <- maxLik( llfInd, gfInd, start = startValFix, activePar = !isFixed,
@@ -422,7 +660,6 @@ hessian( mlFixBfgs )
 logLik( mlFixBfgs )
 maximType( mlFixBfgs )
 nIter( mlFixBfgs )
-nObs( mlFixBfgs )
 nParam( mlFixBfgs )
 returnCode( mlFixBfgs )
 returnMessage( mlFixBfgs )
@@ -431,13 +668,15 @@ logLik( summary( mlFixBfgs ) )
 all.equal( mlghFix[ -c( 5, 6, 9, 10 ) ], mlFixBfgs[ -c( 5, 6, 9, 10, 11 ) ] )
 mlIndFixBfgs <- maxLik( llfInd, start = startValFix, fixed = isFixed,
    method = "BFGS" )
-all.equal( mlFixBfgs[ -9 ], mlIndFixBfgs[ -9 ] )
+all.equal( mlFixBfgs[ -9 ], mlIndFixBfgs[ -c(9,12) ] )
+mlIndFixBfgs[ 12 ]
 mlIndFixBfgs3 <- maxLik( llfInd, start = startValFix, fixed = "mu",
    method = "BFGS" )
 all.equal( mlIndFixBfgs, mlIndFixBfgs3 )
 mlIndFixBfgs4 <- maxLik( llfInd, start = startValFix, fixed = 1,
    method = "BFGS" )
 all.equal( mlIndFixBfgs, mlIndFixBfgs4 )
+nObs( mlIndFixBfgs )
 
 # with analytical gradients
 mlgFixBfgs <- maxLik( llf, gf, start = startValFix, fixed = isFixed,
@@ -452,7 +691,8 @@ summary( mlgFixBfgs )
 all.equal( mlFixBfgs[ -9 ], mlgFixBfgs[ -9 ] )
 mlgIndFixBfgs <- maxLik( llfInd, gfInd, start = startValFix, fixed = isFixed,
    method = "BFGS")
-all.equal( mlgFixBfgs, mlgIndFixBfgs )
+all.equal( mlgFixBfgs[ ], mlgIndFixBfgs[ -12 ] )
+mlgIndFixBfgs[ 12 ]
 mlgIndFixBfgs3 <- maxLik( llfInd, gfInd, start = startValFix, fixed = "mu",
    method = "BFGS" )
 all.equal( mlgIndFixBfgs, mlgIndFixBfgs3 )
@@ -490,7 +730,6 @@ hessian( mlFixNm )
 logLik( mlFixNm )
 maximType( mlFixNm )
 nIter( mlFixNm )
-nObs( mlFixNm )
 nParam( mlFixNm )
 returnCode( mlFixNm )
 returnMessage( mlFixNm )
@@ -499,13 +738,15 @@ logLik( summary( mlFixNm ) )
 all.equal( mlFixBfgs[ -c( 9, 10 ) ], mlFixNm[ -c( 9, 10 ) ] )
 mlIndFixNm <- maxLik( llfInd, start = startValFix, fixed = isFixed,
    method = "NM" )
-all.equal( mlFixNm, mlIndFixNm )
+all.equal( mlFixNm[ ], mlIndFixNm[ -12 ] )
+mlIndFixNm[ 12 ]
 mlIndFixNm3 <- maxLik( llfInd, start = startValFix, fixed = "mu",
    method = "NM" )
 all.equal( mlIndFixNm, mlIndFixNm3 )
 mlIndFixNm4 <- maxLik( llfInd, start = startValFix, fixed = 1,
    method = "NM" )
 all.equal( mlIndFixNm, mlIndFixNm4 )
+nObs( mlIndFixNm )
 
 # with analytical gradients
 mlgFixNm <- maxLik( llf, gf, start = startValFix, fixed = isFixed,
@@ -518,9 +759,10 @@ mlgFixNm4 <- maxLik( llf, gf, start = startValFix, fixed = 1,
 all.equal( mlgFixNm, mlgFixNm4 )
 summary( mlgFixNm )
 all.equal( mlFixNm, mlgFixNm )
-mlgFixNm2 <- maxLik( llfInd, gfInd, start = startValFix, fixed = isFixed,
+mlgIndFixNm <- maxLik( llfInd, gfInd, start = startValFix, fixed = isFixed,
    method = "NM")
-all.equal( mlgFixNm, mlgFixNm2 )
+all.equal( mlgFixNm[ ], mlgIndFixNm[ -12 ] )
+mlgIndFixNm[ 12 ]
 
 # with unused Hessians
 mlghFixNm <- maxLik( llf, gf, hf, start = startValFix, fixed = isFixed,
@@ -552,7 +794,6 @@ hessian( mlFixSann )
 logLik( mlFixSann )
 maximType( mlFixSann )
 nIter( mlFixSann )
-nObs( mlFixSann )
 nParam( mlFixSann )
 returnCode( mlFixSann )
 returnMessage( mlFixSann )
@@ -561,16 +802,19 @@ logLik( summary( mlFixSann ) )
 all.equal( mlFixBfgs[ -c( 9, 10 ) ], mlFixSann[ -c( 9, 10 ) ] )
 mlIndFixSann <- maxLik( llfInd, start = startValFix, fixed = isFixed,
    method = "SANN" )
-all.equal( mlFixSann, mlIndFixSann )
+all.equal( mlFixSann[ ], mlIndFixSann[ -12 ] )
+mlIndFixSann[ 12 ]
+nObs( mlIndFixSann )
 
 # with analytical gradients
 mlgFixSann <- maxLik( llf, gf, start = startValFix, fixed = isFixed,
    method = "SANN" )
 summary( mlgFixSann )
 all.equal( mlFixSann, mlgFixSann )
-mlgFixSann2 <- maxLik( llfInd, gfInd, start = startValFix, fixed = isFixed,
+mlgIndFixSann <- maxLik( llfInd, gfInd, start = startValFix, fixed = isFixed,
    method = "SANN")
-all.equal( mlgFixSann, mlgFixSann2 )
+all.equal( mlgFixSann[ ], mlgIndFixSann[ -12 ] )
+mlgIndFixSann[ 12 ]
 
 # with unused Hessians
 mlghFixSann <- maxLik( llf, gf, hf, start = startValFix, fixed = isFixed,
@@ -604,7 +848,6 @@ hessian( mlBfgsInEq )
 logLik( mlBfgsInEq )
 maximType( mlBfgsInEq )
 nIter( mlBfgsInEq )
-nObs( mlBfgsInEq )
 nParam( mlBfgsInEq )
 returnCode( mlBfgsInEq )
 returnMessage( mlBfgsInEq )
@@ -613,7 +856,9 @@ logLik( summary( mlBfgsInEq ) )
 mlBfgsInEqInd <- maxLik( llfInd, start = startVal, constraints = inEq,
    method = "BFGS" )
 summary( mlBfgsInEqInd )
-all.equal( mlBfgsInEq, mlBfgsInEqInd )
+all.equal( mlBfgsInEq[ ], mlBfgsInEqInd[ -12 ] )
+mlBfgsInEqInd[ 12 ]
+nObs( mlBfgsInEqInd )
 
 # with analytical gradients
 mlgBfgsInEq <- maxLik( llf, gf, start = startVal, constraints = inEq,
@@ -621,7 +866,8 @@ mlgBfgsInEq <- maxLik( llf, gf, start = startVal, constraints = inEq,
 all.equal( mlBfgsInEq, mlgBfgsInEq )
 mlgBfgsInEqInd <- maxLik( llfInd, gfInd, start = startVal, constraints = inEq,
    method = "BFGS" )
-all.equal( mlgBfgsInEqInd, mlgBfgsInEq )
+all.equal( mlgBfgsInEqInd[ -12 ], mlgBfgsInEq[ ] )
+mlgBfgsInEqInd[ 12 ]
 mlgBfgsInEqInd2 <- maxLik( llf, gfInd, start = startVal, constraints = inEq,
    method = "BFGS" )
 all.equal( mlgBfgsInEqInd, mlgBfgsInEqInd2 )
@@ -643,7 +889,6 @@ hessian( mlNmInEq )
 logLik( mlNmInEq )
 maximType( mlNmInEq )
 nIter( mlNmInEq )
-nObs( mlNmInEq )
 nParam( mlNmInEq )
 returnCode( mlNmInEq )
 returnMessage( mlNmInEq )
@@ -653,7 +898,9 @@ all.equal( mlBfgsInEq, mlNmInEq )
 mlNmInEqInd <- maxLik( llfInd, start = startVal, constraints = inEq,
    method = "NM" )
 summary( mlNmInEqInd )
-all.equal( mlNmInEq, mlNmInEqInd )
+all.equal( mlNmInEq[ ], mlNmInEqInd[ -12 ] )
+mlNmInEqInd[ 12 ]
+nObs( mlNmInEqInd )
 
 # with unused analytical gradients
 mlgNmInEq <- maxLik( llf, gf, start = startVal, constraints = inEq,
@@ -678,7 +925,6 @@ hessian( mlSannInEq )
 logLik( mlSannInEq )
 maximType( mlSannInEq )
 nIter( mlSannInEq )
-nObs( mlSannInEq )
 nParam( mlSannInEq )
 returnCode( mlSannInEq )
 returnMessage( mlSannInEq )
@@ -712,7 +958,6 @@ hessian( mlCon )
 logLik( mlCon )
 maximType( mlCon )
 nIter( mlCon )
-nObs( mlCon )
 nParam( mlCon )
 returnCode( mlCon )
 returnMessage( mlCon )
@@ -720,7 +965,9 @@ vcov( mlCon )
 logLik( summary( mlCon ) )
 mlConInd <- maxLik( llfInd, start = startVal, constraints = eqCon )
 summary( mlConInd )
-all.equal( mlCon, mlConInd )
+all.equal( mlCon[], mlConInd[-11] )
+mlConInd[11]
+nObs( mlConInd )
 
 # with analytical gradients
 mlgCon <- maxLik( llf, gf, start = startVal, constraints = eqCon )
@@ -728,12 +975,24 @@ summary( mlgCon )
 all.equal( mlCon[ -c( 5, 6, 7, 9 ) ], mlgCon[ -c( 5, 6, 7, 9 ) ] )
 mlgConInd <- maxLik( llfInd, gfInd, start = startVal, constraints = eqCon )
 all.equal( mlConInd, mlgConInd )
-all.equal( mlgCon, mlgConInd )
+all.equal( mlgCon[], mlgConInd[-11] )
+mlgConInd[11]
+
+# with analytical gradients as attribute
+mlGCon <- maxLik( llfGrad, start = startVal, constraints = eqCon )
+all.equal( mlGCon, mlgCon )
+all.equal( mlGCon, mlCon )
 
 # with analytical gradients and Hessians
 mlghCon <- maxLik( llf, gf, hf, start = startVal, constraints = eqCon )
 all.equal( mlgCon, mlghCon )
 
+# with analytical gradients and Hessians as attributes
+mlGHCon <- maxLik( llfGradHess, start = startVal, constraints = eqCon )
+all.equal( mlGHCon, mlghCon )
+all.equal( mlGHCon, mlCon )
+
+
 ## BHHH method with equality constraints
 mlBhhhCon <- maxLik( llfInd, start = startVal, constraints = eqCon,
    method = "BHHH" )
@@ -747,13 +1006,14 @@ hessian( mlBhhhCon )
 logLik( mlBhhhCon )
 maximType( mlBhhhCon )
 nIter( mlBhhhCon )
-nObs( mlBhhhCon )
 nParam( mlBhhhCon )
 returnCode( mlBhhhCon )
 returnMessage( mlBhhhCon )
 vcov( mlBhhhCon )
 logLik( summary( mlBhhhCon ) )
-all.equal( mlCon[ -c( 5, 6, 7, 9, 10 ) ], mlBhhhCon[ -c( 5, 6, 7, 9, 10 ) ] )
+all.equal( mlCon[ -c( 5, 6, 7, 9, 10 ) ], mlBhhhCon[ -c( 5, 6, 7, 9, 10, 11 ) ] )
+mlBhhhCon[11]
+nObs( mlBhhhCon )
 
 # with analytical gradients
 mlgBhhhCon <- maxLik( llf, gfInd, start = startVal, constraints = eqCon,
@@ -764,11 +1024,25 @@ mlgBhhhConInd <- maxLik( llfInd, gfInd, start = startVal, constraints = eqCon,
    method = "BHHH" )
 all.equal( mlgBhhhCon, mlgBhhhConInd )
 
+# with analytical gradients as attribute
+mlGBhhhCon <- maxLik( llfGradInd, start = startVal, constraints = eqCon,
+   method = "BHHH" )
+summary( mlGBhhhCon )
+all.equal( mlGBhhhCon, mlgBhhhCon )
+all.equal( mlGBhhhCon, mlBhhhCon )
+
 # with analytical gradients and unused Hessians
 mlghBhhhCon <- maxLik( llf, gfInd, hf, start = startVal, constraints = eqCon,
    method = "BHHH" )
 all.equal( mlgBhhhCon, mlghBhhhCon )
 
+# with analytical gradients and unused Hessians as attributes
+mlGHBhhhCon <- maxLik( llfGradHessInd, start = startVal, constraints = eqCon,
+   method = "BHHH" )
+all.equal( mlGHBhhhCon, mlghBhhhCon )
+all.equal( mlGHBhhhCon, mlGBhhhCon )
+
+
 ## BFGS method with equality constraints
 mlBfgsCon <- maxLik( llf, start = startVal, constraints = eqCon,
    method = "BFGS" )
@@ -782,7 +1056,6 @@ hessian( mlBfgsCon )
 logLik( mlBfgsCon )
 maximType( mlBfgsCon )
 nIter( mlBfgsCon )
-nObs( mlBfgsCon )
 nParam( mlBfgsCon )
 returnCode( mlBfgsCon )
 returnMessage( mlBfgsCon )
@@ -792,7 +1065,9 @@ all.equal( mlBfgsCon[ -c( 5, 6, 9, 10 ) ], mlCon[ -c( 5, 6, 9, 10 ) ] )
 mlBfgsConInd <- maxLik( llfInd, start = startVal, constraints = eqCon,
    method = "BFGS" )
 summary( mlBfgsConInd )
-all.equal( mlBfgsCon, mlBfgsConInd )
+all.equal( mlBfgsCon[], mlBfgsConInd[-12] )
+mlBfgsConInd[12]
+nObs( mlBfgsConInd )
 
 # with analytical gradients
 mlgBfgsCon <- maxLik( llf, gf, start = startVal, constraints = eqCon,
@@ -801,7 +1076,8 @@ summary( mlgBfgsCon )
 all.equal( mlBfgsCon, mlgBfgsCon )
 mlgBfgsConInd <- maxLik( llfInd, gfInd, start = startVal, constraints = eqCon,
    method = "BFGS" )
-all.equal( mlgBfgsCon, mlgBfgsConInd )
+all.equal( mlgBfgsCon[], mlgBfgsConInd[-12] )
+mlgBfgsConInd[12]
 
 # with analytical gradients and unused Hessians
 mlghBfgsCon <- maxLik( llf, gf, hf, start = startVal, constraints = eqCon,
@@ -820,7 +1096,6 @@ hessian( mlNmCon )
 logLik( mlNmCon )
 maximType( mlNmCon )
 nIter( mlNmCon )
-nObs( mlNmCon )
 nParam( mlNmCon )
 returnCode( mlNmCon )
 returnMessage( mlNmCon )
@@ -830,7 +1105,9 @@ all.equal( mlNmCon[ -c( 5, 6, 9, 10 ) ], mlCon[ -c( 5, 6, 9, 10 ) ] )
 mlNmConInd <- maxLik( llfInd, start = startVal, constraints = eqCon,
    method = "NM", SUMTTol=0)
 summary( mlNmConInd )
-all.equal( mlNmCon, mlNmConInd )
+all.equal( mlNmCon[], mlNmConInd[-12] )
+mlNmConInd[12]
+nObs( mlNmConInd )
 
 # with unused analytical gradients
 mlgNmCon <- maxLik( llf, gf, start = startVal, constraints = eqCon,
@@ -838,7 +1115,8 @@ mlgNmCon <- maxLik( llf, gf, start = startVal, constraints = eqCon,
 all.equal( mlNmCon, mlgNmCon )
 mlgNmConInd <- maxLik( llfInd, gfInd, start = startVal, constraints = eqCon,
    method = "NM", SUMTTol=0)
-all.equal( mlgNmCon, mlgNmConInd )
+all.equal( mlgNmCon[], mlgNmConInd[-12] )
+mlgNmConInd[12]
 
 # with unused analytical gradients and Hessians
 mlghNmCon <- maxLik( llf, gf, hf, start = startVal, constraints = eqCon,
@@ -858,7 +1136,6 @@ hessian( mlSannCon )
 logLik( mlSannCon )
 maximType( mlSannCon )
 nIter( mlSannCon )
-nObs( mlSannCon )
 nParam( mlSannCon )
 returnCode( mlSannCon )
 returnMessage( mlSannCon )
@@ -877,3 +1154,112 @@ mlSannConCand <- maxLik( llf, start = startVal, constraints = eqCon,
 summary( mlSannConCand )
 all.equal( mlSannConCand, mlSannCon )
 
+
+## test for method "estfun"
+library( sandwich )
+try( estfun( ml ) )
+estfun( mlInd )[ 1:5, ]
+estfun( mlgInd )[ 1:5, ]
+estfun( mlBHHH )[ 1:5, ]
+estfun( mlgBHHH )[ 1:5, ]
+estfun( mlIndBFGS )[ 1:5, ]
+estfun( mlgIndBFGS )[ 1:5, ]
+estfun( mlIndNM )[ 1:5, ]
+estfun( mlgIndNM )[ 1:5, ]
+estfun( mlIndSANN )[ 1:5, ]
+estfun( mlgIndSANN )[ 1:5, ]
+estfun( mlIndFix )[ 1:5, ]
+estfun( mlgIndFix )[ 1:5, ]
+estfun( mlFixBHHH )[ 1:5, ]
+estfun( mlgFixBHHH )[ 1:5, ]
+estfun( mlIndFixBfgs )[ 1:5, ]
+estfun( mlgIndFixBfgs )[ 1:5, ]
+estfun( mlIndFixNm )[ 1:5, ]
+estfun( mlgIndFixNm )[ 1:5, ]
+estfun( mlIndFixSann )[ 1:5, ]
+estfun( mlgIndFixSann )[ 1:5, ]
+estfun( mlBfgsInEqInd )[ 1:5, ]
+estfun( mlgBfgsInEqInd )[ 1:5, ]
+estfun( mlNmInEqInd )[ 1:5, ]
+estfun( mlConInd )[ 1:5, ]
+estfun( mlgConInd )[ 1:5, ]
+estfun( mlBhhhCon )[ 1:5, ]
+estfun( mlgBhhhCon )[ 1:5, ]
+estfun( mlBfgsConInd )[ 1:5, ]
+estfun( mlgBfgsConInd )[ 1:5, ]
+estfun( mlNmConInd )[ 1:5, ]
+estfun( mlgNmConInd )[ 1:5, ]
+
+
+## test for method "bread"
+try( bread( ml ) )
+bread( mlInd )
+bread( mlgInd )
+bread( mlBHHH )
+bread( mlgBHHH )
+bread( mlIndBFGS )
+bread( mlgIndBFGS )
+bread( mlIndNM )
+bread( mlgIndNM )
+bread( mlIndSANN )
+bread( mlgIndSANN )
+bread( mlIndFix )
+bread( mlgIndFix )
+bread( mlFixBHHH )
+bread( mlgFixBHHH )
+bread( mlIndFixBfgs )
+bread( mlgIndFixBfgs )
+bread( mlIndFixNm )
+bread( mlgIndFixNm )
+bread( mlIndFixSann )
+bread( mlgIndFixSann )
+bread( mlBfgsInEqInd )
+bread( mlgBfgsInEqInd )
+bread( mlNmInEqInd )
+bread( mlConInd )
+bread( mlgConInd )
+bread( mlBhhhCon )
+bread( mlgBhhhCon )
+bread( mlBfgsConInd )
+bread( mlgBfgsConInd )
+bread( mlNmConInd )
+bread( mlgNmConInd )
+
+
+## test for method "sandwich"
+try( sandwich( ml ) )
+printSandwich <- function( x ) {
+   print( sandwich( x ) )
+   print( all.equal( sandwich( x ), vcov( x ) ) )
+}
+printSandwich( mlInd )
+printSandwich( mlgInd )
+printSandwich( mlBHHH )
+printSandwich( mlgBHHH )
+printSandwich( mlIndBFGS )
+printSandwich( mlgIndBFGS )
+printSandwich( mlIndNM )
+printSandwich( mlgIndNM )
+printSandwich( mlIndSANN )
+printSandwich( mlgIndSANN )
+printSandwich( mlIndFix )
+printSandwich( mlgIndFix )
+printSandwich( mlFixBHHH )
+printSandwich( mlgFixBHHH )
+printSandwich( mlIndFixBfgs )
+printSandwich( mlgIndFixBfgs )
+printSandwich( mlIndFixNm )
+printSandwich( mlgIndFixNm )
+printSandwich( mlIndFixSann )
+printSandwich( mlgIndFixSann )
+printSandwich( mlBfgsInEqInd )
+printSandwich( mlgBfgsInEqInd )
+printSandwich( mlNmInEqInd )
+printSandwich( mlConInd )
+printSandwich( mlgConInd )
+printSandwich( mlBhhhCon )
+printSandwich( mlgBhhhCon )
+printSandwich( mlBfgsConInd )
+printSandwich( mlgBfgsConInd )
+printSandwich( mlNmConInd )
+printSandwich( mlgNmConInd )
